"""
Poynt Python SDK

Author
Charles Feng <c@poynt.com>
"""
from poynt.api import API
from poynt.business import Business
from poynt.businessuser import BusinessUser
from poynt.catalog import Catalog
from poynt.cloudmessage import CloudMessage
from poynt.customer import Customer
from poynt.hook import Hook
from poynt.order import Order
from poynt.product import Product
from poynt.report import Report
from poynt.store import Store
from poynt.tax import Tax
from poynt.transaction import Transaction
from poynt.business_application import BusinessApplication

application_id = None
env = None
filename = None
key = None
region = None



from poynt import API
from poynt.helpers import json_patch


class Catalog():

    @classmethod
    def get_catalogs(cls, business_id, start_at=None, start_offset=None,
                     end_at=None, limit=None):
        """
        Get all catalogs at a business.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get catalogs created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get catalogs created before this time in seconds
        limit (int, optional): how many catalogs to return (for pagination)
        """

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_catalog(cls, business_id, catalog_id):
        """
        Get a single catalog for a business.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s' % (business_id, catalog_id),
            method='GET'
        )

    @classmethod
    def get_full_catalog(cls, business_id, catalog_id):
        """
        Get a catalog by id with all product details info embedded in
        the Catalog.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s/full' % (business_id, catalog_id),
            method='GET'
        )

    @classmethod
    def create_catalog(cls, business_id, catalog):
        """
        Creates a catalog on a business.

        Arguments:
        business_id (str): the business ID
        catalog (dict): the catalog object
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs' % business_id,
            method='POST',
            json=catalog,
        )

    @classmethod
    def create_full_catalog(cls, business_id, full_catalog):
        """
        Creates a catalog on a business with embedded products.
        This differs from create_catalog as you can create products
        and catalogs at the same time.

        Arguments:
        business_id (str): the business ID
        full_catalog (dict): the full catalog object
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/full' % business_id,
            method='POST',
            json=full_catalog,
        )

    @classmethod
    def update_catalog(cls, business_id, catalog_id, catalog=None, patch=None, no_remove=True):
        """
        Updates a catalog by ID. Can either specify the whole catalog, or an array
        of JSON Patch instructions.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID

        Keyword arguments:
        catalog (dict): the catalog object. this should be a Catalog, not a CatalogWithProduct.
        patch (list of dict): JSON Patch update instructions
        no_remove (boolean, optional): don't remove any keys from old catalog in the patch.
                                       safer this way. defaults to True
        """

        # get patch instructions if only catalog is specified
        if patch is None:
            if catalog is None:
                raise ValueError("Either patch or catalog must be specified")

            old_catalog, status_code = cls.get_catalog(business_id, catalog_id)
            if status_code >= 300 or old_catalog is None:
                raise RuntimeError("Catalog to patch not found")

            patch = json_patch(old_catalog, catalog, no_remove=no_remove)

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s' % (business_id, catalog_id),
            method='PATCH',
            json=patch,
        )

    @classmethod
    def delete_catalog(cls, business_id, catalog_id):
        """
        Deletes a single catalog for a business.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s' % (business_id, catalog_id),
            method='DELETE'
        )

    @classmethod
    def get_category(cls, business_id, catalog_id, category_id):
        """
        Get a single category in a catalog for a business.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        category_id (str): the category ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s/categories/%s' % (
                business_id, catalog_id, category_id),
            method='GET'
        )

    @classmethod
    def create_category(cls, business_id, catalog_id, category):
        """
        Creates a category on a catalog on a business.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        category (dict): the category object
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s/categories' % (
                business_id, catalog_id),
            method='POST',
            json=category,
        )

    @classmethod
    def lookup_categories(cls, business_id, catalog_id, category_ids):
        """
        Gets multiple categories on a catalog by IDs.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        category_ids (list of str): a list of category ids
        """

        params = {
            'ids': category_ids
        }

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s/categories/lookup' % (
                business_id, catalog_id),
            method='GET',
            params=params,
        )

    @classmethod
    def delete_category(cls, business_id, catalog_id, category_id):
        """
        Deletes a category by ID.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        category_id (str): the category ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s/categories/%s' % (
                business_id, catalog_id, category_id),
            method='DELETE'
        )

    @classmethod
    def update_category(cls, business_id, catalog_id, category_id, category=None, patch=None, no_remove=True):
        """
        Updates a category by ID. Can either specify the whole category, or an array
        of JSON Patch instructions.

        Arguments:
        business_id (str): the business ID
        catalog_id (str): the catalog ID
        category_id (str): the category ID

        Keyword arguments:
        category (dict): the category object
        patch (list of dict): JSON Patch update instructions
        no_remove (boolean, optional): don't remove any keys from old category in the patch.
                                       safer this way. defaults to True
        """

        # get patch instructions if only category is specified
        if patch is None:
            if category is None:
                raise ValueError("Either patch or category must be specified")

            old_category, status_code = cls.get_category(
                business_id, catalog_id, category_id)
            if status_code >= 300 or old_category is None:
                raise RuntimeError("Category to patch not found")

            patch = json_patch(old_category, category, no_remove=no_remove)

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/catalogs/%s/categories/%s' % (
                business_id, catalog_id, category_id),
            method='PATCH',
            json=patch,
        )



def json_patch(from_obj, to_obj, ignore_keys=None, only_keys=None, no_remove=False):
    """
    Creates a JSON patch diff between two objects.

    Arguments:
    from_obj (dict): from object, usually the existing object returned by API
    to_obj (dict): to object, usually the new object to return

    Keyword arguments:
    ignore_keys (list of str, optional): whether to ignore any keys (e.g. not patch them)
    only_keys (list of str, optional): only patch keys from this whitelist
    no_remove (bool, optional): don't remove any keys when patching (in case you forgot
                                to specify certain keys in the object)
    """

    ops = []

    for key in to_obj:
        # filter keys based on ignore_keys, only_keys
        if ignore_keys is not None and key in ignore_keys:
            continue
        if only_keys is not None and key not in only_keys:
            continue

        # when key is not in from but in to, use add op
        if key not in from_obj or from_obj[key] is None:
            ops.append({
                'op': 'add',
                'path': '/%s' % key,
                'value': to_obj[key]
            })
        elif from_obj[key] != to_obj[key]:
            ops.append({
                'op': 'replace',
                'path': '/%s' % key,
                'value': to_obj[key]
            })

    if no_remove is False:
        for key in from_obj:
            # filter keys based on ignore_keys, only_keys
            if ignore_keys is not None and key in ignore_keys:
                continue
            if only_keys is not None and key not in only_keys:
                continue

            # when key is in from and not in to, use remove op
            if key not in to_obj or to_obj[key] is None:
                ops.append({
                    'op': 'remove',
                    'path': '/%s' % key
                })

    return ops



from poynt import API


class CloudMessage():

    @classmethod
    def send_raw_cloud_message(cls, message):
        """
        Sends a cloud message by specifying the entire message object.

        Arguments:
        message (dict): the full cloud message object
        """

        api = API.shared_instance()
        return api.request(
            url='/cloudMessages',
            method='POST',
            json=message,
        )

    @classmethod
    def send_cloud_message(cls, business_id=None, store_id=None,
                           class_name=None, package_name=None, device_id=None,
                           serial_number=None, data='{}', collapse_key=None,
                           ttl=900):
        """
        Send a message from the cloud to your application running at a Poynt terminal.

        Keyword arguments:
        business_id (str): the business ID
        store_id (str): the store ID
        class_name (str): the class name of the receiver in your app
        package_name (str): the package name of your app
        device_id (str): the device ID
        serial_number (str): the serial number
        data (str, optional): the data to send. defaults to {}
        collapse_key (str, optional): dedupe messages on this key
        ttl (int, optional): how long until the cloud message expires. defaults
                             to 900 seconds
        """

        message = {
            'businessId': business_id,
            'storeId': store_id,
            'deviceId': device_id,
            'recipient': {
                'className': class_name,
                'packageName': package_name,
            },
        }

        if serial_number is not None:
            message['serialNum'] = serial_number
        if collapse_key is not None:
            message['collapseKey'] = collapse_key
        if ttl is not None:
            message['ttl'] = ttl
        if data is not None:
            message['data'] = data

        return cls.send_raw_cloud_message(message)



from poynt import API


class Order():

    @classmethod
    def get_orders(cls, business_id, start_at=None, start_offset=None,
                   end_at=None, limit=None, card_number_first_6=None,
                   card_number_last_4=None, card_expiration_month=None,
                   card_expiration_year=None, card_holder_first_name=None,
                   card_holder_last_name=None, store_id=None, link=None):
        """
        Get all orders at a business by various criteria.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get orders created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get orders created before this time in seconds
        limit (int, optional): how many orders to return (for pagination)
        card_number_first_6 (str, optional): return orders with card numbers starting with this
        card_number_last_4 (str, optional): return orders with card numbers ending with this
        card_expiration_month (str, optional): return orders with this card expiration month
        card_expiration_year (str, optional): return orders with this card expiration year
        card_holder_first_name (str, optional): return orders with first name matching this
        card_holder_last_name (str, optional): return orders with last name matching this
        store_id (str, optional): return orders from this store
        link (str, optional): HATEOAS link to fetch next page of orders
        """

        if link:
            api = API.shared_instance()
            return api.request(
                url=link,
                method='GET',
            )

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit
        if card_number_first_6 is not None:
            params['cardNumberFirst6'] = card_number_first_6
        if card_number_last_4 is not None:
            params['cardNumberLast4'] = card_number_last_4
        if card_expiration_month is not None:
            params['cardExpirationMonth'] = card_expiration_month
        if card_expiration_year is not None:
            params['cardExpirationYear'] = card_expiration_year
        if card_holder_first_name is not None:
            params['cardHolderFirstName'] = card_holder_first_name
        if card_holder_last_name is not None:
            params['cardHolderLastName'] = card_holder_last_name
        if store_id is not None:
            params['storeId'] = store_id

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/orders' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_order(cls, business_id, order_id):
        """
        Get a single order at a business.

        Arguments:
        business_id (str): the business ID
        order_id (str): the order ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/orders/%s' % (business_id, order_id),
            method='GET'
        )



from poynt import API


class Hook():

    @classmethod
    def get_hooks(cls, business_id):
        """
        Gets a list of hooks currently subscribed to.

        Arguments:
        business_id (str): use a merchant business ID to see what webhooks your app
                           is subscribed to for that merchant. use your own organization
                           ID to see your app billing webhooks, etc.
        """

        params = {
            'businessId': business_id
        }

        api = API.shared_instance()
        return api.request(
            url='/hooks',
            method='GET',
            params=params,
        )

    @classmethod
    def create_hook(cls, business_id, delivery_url, secret=None, event_type=None,
                    event_types=None):
        """
        Subscribes to a webhook.

        Arguments:
        business_id (str): the business ID to subscribe to a hook for. Use a merchant
                           business ID to subscribe to their e.g. transaction hooks;
                           use your own organization ID to subscribe to app billing, etc.
        delivery_url (str): the URL to deliver webhooks to.

        Keyword arguments:
        secret (str, optional): used to sign the webhook event, so you can verify.
        event_type (str, optional): a single event type to subscribe to webhooks for.
        event_types (list of str, optional): a list of event types to subscribe to webhooks for.
        """

        api = API.shared_instance()

        json = {
            'applicationId': api.application_id,
            'businessId': business_id,
            'deliveryUrl': delivery_url,
        }
        if secret is not None:
            json['secret'] = secret
        if event_types is not None:
            json['eventTypes'] = event_types
        elif event_type is not None:
            json['eventTypes'] = [event_type]

        return api.request(
            url='/hooks',
            method='POST',
            json=json,
        )

    @classmethod
    def get_hook(cls, hook_id):
        """
        Gets a hook by ID.

        Arguments:
        hook_id (str): hook ID
        """

        api = API.shared_instance()
        return api.request(
            url='/hooks/%s' % hook_id,
            method='GET'
        )

    @classmethod
    def delete_hook(cls, hook_id):
        """
        Deletes a hook.

        Arguments:
        hook_id (str): hook ID
        """

        api = API.shared_instance()
        return api.request(
            url='/hooks/%s' % hook_id,
            method='DELETE'
        )



import jwt
import requests
import time
import uuid

import poynt

poynt_urls = {
    'dev': 'https://services-ci.poynt.net',
    'ci': 'https://services-ci.poynt.net',
    'st': 'https://services-st.poynt.net',
    'prod': 'https://services.poynt.net',
    'eu': 'https://services-eu.poynt.net'
}

poynt_web_urls = {
    'dev': 'http://local.poynt.net/api/services',
    'ci': 'https://ci.poynt.net/api/services',
    'st': 'https://st.poynt.net/api/services',
    'prod': 'https://eu.poynt.net/api/services',
    'eu': 'https://poynt.net/api/services'
}

shared_instance = None


class API(object):

    def __init__(self, key=None, filename=None, env=None, application_id=None, region=None):
        """
        Instantiates the API class, which creates and authenticates requests
        to the Poynt API.

        Keyword arguments:
        key (str): the PEM-encoded private key associated with the application
        filename (str): the file where the PEM-encoded private key is stored
        env (str, optional): optional environment param. Defaults to "prod"
        application_id (str): your app's application ID
        """

        if key is not None:
            self.key = key
        elif filename is not None:
            f = open(filename, 'r')
            self.key = f.read()
        else:
            raise ValueError(
                "Either poynt.key or poynt.filename must be specified")

        if application_id is None:
            raise ValueError(
                "Application ID must be specified")

        self.application_id = application_id
        self.api_version = '1.2'

        if region in poynt_urls:
            self.region = region
            self.api_root = poynt_urls[self.region]
            self.web_root = poynt_web_urls[self.region]
        else:
            if env in poynt_urls:
                self.env = env
            else:
                self.env = 'prod'
            self.api_root = poynt_urls[self.env]
            self.web_root = poynt_web_urls[self.env]

        self.session = requests.Session()
        self.access_token = None
        self.expires_at = None

    def _request_headers(self):
        """
        Private method that generates request headers for Poynt API requests.
        """

        headers = {
            'User-Agent': 'python-sdk-%s' % self.application_id,
            'Api-Version': self.api_version,
            'Poynt-Request-Id': str(uuid.uuid4()),
        }

        if self.access_token:
            headers['Authorization'] = 'Bearer %s' % self.access_token

        return headers

    def _naked_request(self, method, url, json=None, headers=None, form=None,
                       params=None, app='API'):
        """
        Private method that makes a request to Poynt APIs without error protection.
        You shouldn't use this in your SDK methods.

        Arguments:
        method (str): the request method, e.g. GET, POST, PATCH, etc.
        url (str): the request relative URL, e.g. /businesses

        Keyword arguments:
        json (dict, optional): request data to send, encoded as JSON
        headers (dict, optional): request headers to send
        form (dict, optional): request data to send, encoded as a form. Use this or json
        params (dict, optional): request params to send in the querystring
        app (str, optional): API or WEB. defaults to API
        """

        headers = self._request_headers()
        method = method or 'GET'
        url_root = self.web_root if app is 'WEB' else self.api_root

        if form:
            r = self.session.request(
                method,
                url=url_root + url,
                data=form,
                headers=headers,
                params=params,
            )
        else:
            r = self.session.request(
                method,
                url=url_root + url,
                json=json,
                headers=headers,
                params=params,
            )

        try:
            json = r.json()
            return json, r.status_code
        except ValueError:
            return None, r.status_code

    def _access_token_is_expired(self):
        """
        Private method that tells you if your access token doesn't exist,
        or is expired.
        """

        if self.access_token is None:
            return True

        my_jwt = jwt.decode(self.access_token, verify=False)

        if my_jwt['exp'] < int(round(time.time())) + 60:
            return True

        return False

    def _authenticate(self):
        """
        Authenticates with Poynt API using your application private key.
        """

        now = int(round(time.time() * 1000))
        my_jwt = jwt.encode({
            'aud': self.api_root,
            'iss': self.application_id,
            'sub': self.application_id,
            'iat': now,
            'jti': str(uuid.uuid4()),
            'exp': now + 1000 * 60 * 60 * 24 * 365,
        }, self.key, algorithm='RS256')

        json, status_code = self._naked_request(
            method='POST',
            url='/token',
            form={
                'grantType': 'urn:ietf:params:oauth:grant-type:jwt-bearer',
                'assertion': my_jwt
            }
        )

        if json is not None:
            self.access_token = json['accessToken']
            self.expires_at = now + json['expiresIn']

    def request(self, method, url, json=None, headers=None, form=None,
                params=None, force_token_refresh=False, app='API'):
        """
        Makes a request against API service. Refreshes your access token if
        necessary. Use this for all requests in the SDK!

        Arguments:
        method (str): the request method, e.g. GET, POST, PATCH, etc.
        url (str): the request relative URL, e.g. /businesses

        Keyword arguments:
        json (dict, optional): request data to send, encoded as JSON
        headers (dict, optional): request headers to send
        form (dict, optional): request data to send, encoded as a form. Use this or JSON
        params (dict, optional): request params to send in the querystring
        force_token_refresh (bool, optional): whether to force refresh the access token or not
        app (str, optional): API or WEB. defaults to API
        """

        if self._access_token_is_expired() or force_token_refresh:
            self._authenticate()

        json, status_code = self._naked_request(
            method=method,
            url=url,
            json=json,
            headers=headers,
            form=form,
            params=params,
            app=app,
        )

        if status_code is 401 and json is not None and json['code'] is 'INVALID_ACCESS_TOKEN':
            return self.request(
                method=method,
                url=url,
                json=json,
                headers=headers,
                form=form,
                params=params,
                force_token_refresh=True,
                app=app,
            )

        return json, status_code

    @classmethod
    def shared_instance(cls):
        """
        Gets a shared instance of API for making requests, so we can use the
        same access token globally in our application.
        """

        global shared_instance

        if shared_instance is not None:
            return shared_instance

        shared_instance = cls(
            key=poynt.key,
            filename=poynt.filename,
            env=poynt.env,
            application_id=poynt.application_id,
            region=poynt.region
        )

        return shared_instance



from poynt import API


class Business():

    @classmethod
    def get_business(cls, business_id):
        """
        Gets a business by ID.

        Arguments:
        business_id (str): the business ID to get
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s' % business_id,
            method='GET'
        )

    @classmethod
    def get_business_by_device_id(cls, device_id):
        """
        Get a business by a device id.

        Arguments:
        device_id (str): the device ID to get a business for
        """

        params = {
            'storeDeviceId': device_id
        }

        api = API.shared_instance()
        return api.request(
            url='/businesses',
            method='GET',
            params=params
        )



from poynt import API
from poynt.helpers import json_patch


class Tax():

    @classmethod
    def get_taxes(cls, business_id, start_at=None, start_offset=None,
                  end_at=None, limit=None):
        """
        Get a list of taxes at a business.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get taxes created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get taxes created before this time in seconds
        limit (int, optional): how many taxes to return (for pagination)
        """

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/taxes' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_tax(cls, business_id, tax_id):
        """
        Get a single tax for a business.

        Arguments:
        business_id (str): the business ID
        tax_id (str): the tax ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/taxes/%s' % (business_id, tax_id),
            method='GET'
        )

    @classmethod
    def create_tax(cls, business_id, tax):
        """
        Creates a tax on a business.

        Arguments:
        business_id (str): the business ID
        tax (dict): the full tax object
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/taxes' % business_id,
            method='POST',
            json=tax,
        )

    @classmethod
    def delete_tax(cls, business_id, tax_id):
        """
        Deletes a tax.

        Arguments:
        business_id (str): the business ID
        tax_id (str): the tax ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/taxes/%s' % (business_id, tax_id),
            method='DELETE'
        )

    @classmethod
    def update_tax(cls, business_id, tax_id, tax=None, patch=None, no_remove=True):
        """
        Updates a tax by ID. Can either specify the whole tax, or an array
        of JSON Patch instructions.

        Arguments:
        business_id (str): the business ID
        tax_id (str): the tax ID

        Keyword arguments:
        tax (dict): the full tax object
        patch (list of dict): JSON Patch update instructions
        no_remove (boolean, optional): don't remove any keys from old tax in the patch.
                                       safer this way. defaults to True
        """

        # get patch instructions if only tax is specified
        if patch is None:
            if tax is None:
                raise ValueError("Either patch or tax must be specified")

            old_tax, status_code = cls.get_tax(business_id, tax_id)
            if status_code >= 300 or old_tax is None:
                raise RuntimeError("Tax to patch not found")

            patch = json_patch(old_tax, tax, no_remove=no_remove)

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/taxes/%s' % (business_id, tax_id),
            method='PATCH',
            json=patch,
        )



from poynt import API


class Customer():

    @classmethod
    def get_customers(cls, business_id, start_at=None, start_offset=None,
                      end_at=None, limit=None, card_number_first_6=None,
                      card_number_last_4=None, card_expiration_month=None,
                      card_expiration_year=None, card_holder_first_name=None,
                      card_holder_last_name=None):
        """
        Get all customers of a business by various criteria.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get customers created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get customers created before this time in seconds
        limit (int, optional): how many customers to return (for pagination)
        card_number_first_6 (str, optional): return customers with card numbers starting with this
        card_number_last_4 (str, optional): return customers with card numbers ending with this
        card_expiration_month (str, optional): return customers with this card expiration month
        card_expiration_year (str, optional): return customers with this card expiration year
        card_holder_first_name (str, optional): return customers with first name matching this
        card_holder_last_name (str, optional): return customers with last name matching this
        """

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit
        if card_number_first_6 is not None:
            params['cardNumberFirst6'] = card_number_first_6
        if card_number_last_4 is not None:
            params['cardNumberLast4'] = card_number_last_4
        if card_expiration_month is not None:
            params['cardExpirationMonth'] = card_expiration_month
        if card_expiration_year is not None:
            params['cardExpirationYear'] = card_expiration_year
        if card_holder_first_name is not None:
            params['cardHolderFirstName'] = card_holder_first_name
        if card_holder_last_name is not None:
            params['cardHolderLastName'] = card_holder_last_name

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/customers' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_customer(cls, business_id, customer_id):
        """
        Get a single customer at a business.

        Arguments:
        business_id (str): the business ID
        customer_id (str): the customer ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/customers/%s' % (business_id, customer_id),
            method='GET'
        )



from poynt import API


class Report():

    @classmethod
    def get_reports(cls, business_id, store_id=None, device_id=None, timezone=None, page=1, limit=25, report_type=None,
                    user_id=None, including_date=None):
        """
        Get reports at a business.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        store_id (str, optional): the store ID
        device_id (str, optional): the device ID
        timezone (str, optional): the timezone
        page (int, optional): the page of reports to load. defaults to 1
        limit (int, optional): the number of reports to load per page. defaults to 25
        report_type (str, optional): report types to filter by
        user_id (str, optional): user id to filter by
        including_date (str, optional): show only reports whose time range overlaps to this date
        """

        params = {}
        if store_id is not None:
          params['storeId'] = store_id
        if device_id is not None:
          params['deviceId'] = device_id
        if timezone is not None:
          params['timezone'] = timezone
        if page is not None:
          params['page'] = page
        if limit is not None:
          params['limit'] = limit
        if report_type is not None:
          params['reportType'] = report_type
        if user_id is not None:
          params['userId'] = user_id
        if including_date is not None:
          params['includingDate'] = including_date

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/reports' % business_id,
            method='GET',
            params=params,
            app='WEB',
        )

    @classmethod
    def create_report(cls, business_id, report_type, start, end, store_id=None,
                      tid=None, employee_id=None, employee_name=None):
        """
        Creates a report at a business.

        Arguments:
        business_id (str): the business ID
        report_type (str): the report type
        start (str): ISO8601 formatted report start date/time
        end (str): ISO8601 formatted report end date/time

        Keyword arguments:
        store_id (str, optional): the store ID
        tid (str, optional): the tid
        employee_id (str, optional): the employee ID
        employee_name (str, optional): the employee name
        """

        params = {}
        params['type'] = report_type
        params['start'] = start
        params['end'] = end
        if store_id is not None:
          params['storeId'] = store_id
        if tid is not None:
          params['tid'] = tid
        if employee_id is not None:
          params['employeeId'] = employee_id
        if employee_name is not None:
          params['employeeName'] = employee_name

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/reports' % business_id,
            method='POST',
            json=params,
            app='WEB',
        )



from poynt import API

class BusinessApplication():
    """
    A Class providing methods to fetch business application information

    """

    @classmethod
    def get_business_application(cls, business_id):
        """
        Gets a business application

        Arguments:
        business_id (str): the business ID of the business
        """
        api = API.shared_instance()
        return api.request(
            url='/businesses/' + business_id + '/payfac-application',
            method='GET',
            app='WEB',
        )

    @classmethod
    def get_business_account(cls, business_id):
        """
        Get account information related to the business

        Arguments:
        business_id (str): the business ID of the business
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/' + business_id + '/payfac-application/account',
            method='GET',
            app='WEB',
        )

    @classmethod
    def get_business_orders(cls, business_id):
        """
        Gets the orders attached to business application

        Arguments:
        business_id (str): the business ID of the business
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/' + business_id +
                '/payfac-application/orders',
            method='GET',
            app='WEB',
        )

    @classmethod
    def get_business_application_status(cls, business_id):
        """
        Get the status of business application

        Arguments:
        business_id (str): the business ID of the business
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/' + business_id +
                '/payfac-application/status',
            method='GET',
            app='WEB',
        )

    @classmethod
    def get_business_application_profile(cls, business_id):
        """
        Get business profile info

        Arguments:
        business_id (str): the business ID of the business
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/' + business_id +
                '/payfac-application/profile',
            method='GET',
            app='WEB',
        )



from poynt import API


class Transaction():

    @classmethod
    def get_transactions(cls, business_id, start_at=None, start_offset=None,
                         end_at=None, limit=None, card_number_first_6=None,
                         card_number_last_4=None, card_expiration_month=None,
                         card_expiration_year=None, card_holder_first_name=None,
                         card_holder_last_name=None, store_id=None, device_id=None,
                         search_key=None, action=None, status=None, transaction_ids=None,
                         auth_only=None, unsettled_only=None, credit_debit_only=None,
                         link=None):
        """
        Get all transactions at a business by various criteria.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get txns created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get txns created before this time in seconds
        limit (int, optional): how many txns to return (for pagination)
        card_number_first_6 (str, optional): return txns with card numbers starting with this
        card_number_last_4 (str, optional): return txns with card numbers ending with this
        card_expiration_month (str, optional): return txns with this card expiration month
        card_expiration_year (str, optional): return txns with this card expiration year
        card_holder_first_name (str, optional): return txns with first name matching this
        card_holder_last_name (str, optional): return txns with last name matching this
        store_id (str, optional): return txns from this store
        device_id (str, optional): return txns from this device
        search_key (str, optional): instead of specifying which exact field to look at, the
                                    client can simply pass this search key and the server will
                                    look at various different fields,
        action (str, optional): only fetch txns with this action
        status (str, optional): only fetch txns with this status
        transaction_ids (str, optional): only fetch txns matching these ids (comma separated)
        auth_only (bool, optional): only fetch auth only txns
        unsettled_only (bool, optional): only fetch unsettled txns
        credit_debit_only (bool, optional): only fetch credit/debit txns
        link (str, optional): HATEOAS link to fetch next page of transactions
        """

        if link:
          api = API.shared_instance()
          return api.request(
              url=link,
              method='GET',
          )

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit
        if card_number_first_6 is not None:
            params['cardNumberFirst6'] = card_number_first_6
        if card_number_last_4 is not None:
            params['cardNumberLast4'] = card_number_last_4
        if card_expiration_month is not None:
            params['cardExpirationMonth'] = card_expiration_month
        if card_expiration_year is not None:
            params['cardExpirationYear'] = card_expiration_year
        if card_holder_first_name is not None:
            params['cardHolderFirstName'] = card_holder_first_name
        if card_holder_last_name is not None:
            params['cardHolderLastName'] = card_holder_last_name
        if store_id is not None:
            params['storeId'] = store_id
        if device_id is not None:
            params['deviceId'] = device_id
        if search_key is not None:
            params['searchKey'] = search_key
        if action is not None:
            params['action'] = action
        if status is not None:
            params['status'] = status
        if transaction_ids is not None:
            params['transactionIds'] = transaction_ids
        if auth_only is not None:
            params['authOnly'] = auth_only
        if unsettled_only is not None:
            params['unsettledOnly'] = unsettled_only
        if credit_debit_only is not None:
            params['creditDebitOnly'] = credit_debit_only

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/transactions' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_transaction(cls, business_id, transaction_id):
        """
        Get a single transaction at a business.

        Arguments:
        business_id (str): the business ID
        transaction_id (str): the transaction ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/transactions/%s' % (
                business_id, transaction_id),
            method='GET'
        )



from poynt import API
from poynt.helpers import json_patch


class Product():

    @classmethod
    def get_products(cls, business_id, start_at=None, start_offset=None,
                     end_at=None, limit=None):
        """
        Get a list of products at a business.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get products created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get products created before this time in seconds
        limit (int, optional): how many products to return (for pagination)
        """

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_products_summary(cls, business_id, start_at=None, start_offset=None,
                             end_at=None, limit=None):
        """
        Get a list of product summaries at a business. Product summaries contain
        product shortCode, price, businessId, name, and id.

        Arguments:
        business_id (str): the business ID

        Keyword arguments:
        start_at (int, optional): get products created after this time in seconds
        start_offset (int, optional): the numeric offset to start the list (for pagination)
        end_at (int, optional): get products created before this time in seconds
        limit (int, optional): how many products to return (for pagination)
        """

        params = {}
        if start_at is not None:
            params['startAt'] = start_at
        if start_offset is not None:
            params['startOffset'] = start_offset
        if end_at is not None:
            params['endAt'] = end_at
        if limit is not None:
            params['limit'] = limit

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products/summary' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def lookup_products(cls, business_id, product_ids):
        """
        Get a list of products by ID.

        Arguments:
        business_id (str): the business ID
        product_ids (list of str): a list of product ids
        """

        params = {
            'ids': product_ids
        }

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products/lookup' % business_id,
            method='GET',
            params=params,
        )

    @classmethod
    def get_product(cls, business_id, product_id):
        """
        Get a single product for a business.

        Arguments:
        business_id (str): the business ID
        product_id (str): the product ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products/%s' % (business_id, product_id),
            method='GET'
        )

    @classmethod
    def create_product(cls, business_id, product):
        """
        Creates a product on a business.

        Arguments:
        business_id (str): the business ID
        product (dict): the full product object
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products' % business_id,
            method='POST',
            json=product,
        )

    @classmethod
    def delete_product(cls, business_id, product_id):
        """
        Deactivates a product. Deactivated products will be removed from all
        catalog references.

        Arguments:
        business_id (str): the business ID
        product_id (str): the product ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products/%s' % (business_id, product_id),
            method='DELETE'
        )

    @classmethod
    def update_product(cls, business_id, product_id, product=None, patch=None, no_remove=True):
        """
        Updates a product by ID. Can either specify the whole product, or an array
        of JSON Patch instructions.

        Arguments:
        business_id (str): the business ID
        product_id (str): the product ID

        Keyword arguments:
        product (dict): the full product object
        patch (list of dict): JSON Patch update instructions
        no_remove (boolean, optional): don't remove any keys from old product in the patch.
                                       safer this way. defaults to True
        """

        # get patch instructions if only product is specified
        if patch is None:
            if product is None:
                raise ValueError("Either patch or product must be specified")

            old_product, status_code = cls.get_product(business_id, product_id)
            if status_code >= 300 or old_product is None:
                raise RuntimeError("Product to patch not found")

            patch = json_patch(old_product, product, no_remove=no_remove)

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/products/%s' % (business_id, product_id),
            method='PATCH',
            json=patch,
        )



from poynt import API


class Store():

    @classmethod
    def get_store(cls, business_id, store_id):
        """
        Gets a store by ID.

        Arguments:
        business_id (str): the business ID
        store_id (str): the store ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/stores/%s' % (business_id, store_id),
            method='GET'
        )



#!/usr/bin/env python

"""
Poynt

The official Poynt Python SDK. Connect to the Poynt API, get/create business
information, and send cloud messages to your terminal app.
"""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='poynt',
    version='0.0.16',
    description='The official Poynt Python SDK.',
    long_description=__doc__,
    author='Charles Feng',
    author_email='c@poynt.com',
    url='https://github.com/poynt/poynt-python',
    license='MIT',
    packages=['poynt'],
    install_requires=['requests', 'pyjwt'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from poynt import API


class BusinessUser():

    @classmethod
    def get_business_users(cls, business_id):
        """
        Get all users at a business.

        Arguments:
        business_id (str): the business ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/businessUsers' % business_id,
            method='GET'
        )

    @classmethod
    def get_business_user(cls, business_id, business_user_id):
        """
        Get a single user at a business.

        Arguments:
        business_id (str): the business ID
        business_user_id (str): the user ID
        """

        api = API.shared_instance()
        return api.request(
            url='/businesses/%s/businessUsers/%s' % (
                business_id, business_user_id),
            method='GET'
        )
